declare const SzyxPush: ISzyxPush;
export default SzyxPush;


export declare enum OsType {
    ios = 1,
    android = 2,
    web = 3,
    windows = 4,
    mac = 5,
    wx = 6,
    unknown = -1,
}

export const enum MsgType {
    unknown = -1,
    text = 1,
    voice = 2,
    image = 3,
    location = 4,
    video = 5,
    file = 6,
    at = 7,
    other = 8,
    log = 9,

}

export declare type MessageListener = (messageType: MsgType, message: Message) => void;

/**
 * 当messageType为text的时候，返回这个
 */
export declare interface TextMessage {
    text: string
}

export declare interface PictureMessage {
    url: string
    thumbnail: string
    thumbnailWidth: number
    thumbnailHeight: number
}

export declare interface Message {
    /**
     * 租户id
     */
    tenantNo: string;
    messageType: MsgType;
    messageId: string;
    target: string;
    fromClientId: string;
    toClientId: string;
    timestamp: number;
    message: string | TextMessage | PictureMessage;
}


export declare interface ISzyxPush {

    setBaseUrl(url: string)

    install(app, tenantNo: string, osType: OsType, url?: string): Promise<void>;


    /**
     * 初始化方法，建议放在main.js中
     * @param tenantNo 租户id，在平台获取
     * @param osType 设备类型 {@link OsType}
     * @param url 服务端地址，可以不传
     */
    init(tenantNo: string, osType: OsType, url?: string): Promise<void>;

    register(userId: string): Promise<void>;

    /**
     *添加消息回调，到达的消息统一分发入口
     * @param callback
     * @return callbackId 页面关闭时记得用 {@link unMessage}注销
     */
    onMessage(callback: MessageListener): number;

    /**
     *注销消息回调
     * @param callId 由{@link onMessage}返回
     */
    unMessage(callId): void;

    sendTextMsg(userId: string, text: string, describe?: string): Promise<Message>;
}
