import Fingerprint2 from 'fingerprintjs2';
import axios from 'axios';
import Stomp from "stompjs";


export var OsType;
(function (OsType) {
    OsType[OsType["unknown"] = -1] = "unknown";
    OsType[OsType["ios"] = 1] = "ios";
    OsType[OsType["android"] = 2] = "android";
    OsType[OsType["web"] = 3] = "web";
    OsType[OsType["windows"] = 4] = "windows";
    OsType[OsType["mac"] = 5] = "mac";
    OsType[OsType["wx"] = 6] = "wx";
})(OsType || (OsType = {}));


export const MsgType = {
    unknown: -1, text: 1, voice: 2, image: 3, location: 4, video: 5, file: 6, at: 7, other: 8, log: 9,
}

class SzyxPushClient {


    constructor() {
        this.mapCall = new Map();
        this._mapMsg = new Map();

        this._host = null;
        this._port = null;
        this._deviceType = null;

        this._user = null;

        this._client = null;
        this.socketOpen = false;
        this.ws = {
            send: this.sendSocketMessage
        }
    }

    guid() {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
            const r = Math.random() * 16 | 0, v = c === 'x' ? r : (r & 0x3 | 0x8);
            return v.toString(16);
        });
    }

    _parseMsg(message) {

        let msg = message.message
        try {
            msg = JSON.parse(message.message)
            // eslint-disable-next-line no-empty
        } catch (e) {

        }

        return {...message, message: msg}
    }

    setHostAndPort(host, port) {
        this._host = host
        this._port = port
    }

    setDeviceType(deviceType) {
        this._deviceType = deviceType
    }

    onMessage(call) {
        let callId = Math.random()
        if (this.mapCall.has(callId)) callId = Math.random()
        this.mapCall.set(callId, call)
        return callId
    }

    unMessage(callId) {
        this.mapCall.delete(callId)
    }

    sendTextMsg(userId, text, describe) {
        return this.sendMessage({
            toClientId: userId, messageType: MsgType.file, message: `{"text":"${text}"}`
        }, describe)
    }

    sendMessage(msg, describe) {
        return new Promise((resolve, reject) => {
            let messageId = this.guid()
            let message = {
                messageId: messageId,
                target: this._user._userId,
                fromClientId: this._user._userId,
                tenantNo: this._user._tenantNo,
                timestamp: Date.now(),
                packetType: "SEND", ...msg
            }
            if (describe)
                message.describe = describe
            this._mapMsg.set(messageId, {call: resolve, message: message})
            this._client.send(`/topic/server`, {}, JSON.stringify(message))
            setTimeout(() => {
                if (this._mapMsg.has(messageId)) reject({
                    msg: '发送消息失败', content: this._parseMsg(message)
                })
                this._mapMsg.delete(messageId)
            }, 500)
        })
    }

    sendSocketMessage(msg) {
        wx.sendSocketMessage({ // 通过 WebSocket连接发送数据
            data: msg
        })
    }


    _connectClient(resolve, reject) {

        this._client.debug = null
        //进行连接
        this._client.connect(this._user._userName, this._user._password, () => {
            resolve()
            this._client.subscribe("/topic/" + this._user._userId, msg => {
                const message = JSON.parse(msg.body)
                if (message.packetType === 'SEND') {
                    this._client.send(`/topic/server`, {}, JSON.stringify({
                        packetType: 'SENDACK', messageId: message.messageId
                    }))
                    for (const call of this.mapCall.values()) {
                        call(message.messageType, this._parseMsg(message));
                    }
                } else if (message.packetType === 'SENDACK') {
                    const m = this._mapMsg.get(message.messageId)
                    if (m) m.call(this._parseMsg(m.message))
                    this._mapMsg.delete(message.messageId)
                }
            }, error => reject(`订阅消息失败::${error}`));
        }, error => reject(`连接服务器失败::${error}`), '/');
    }

    connect(user) {
        this._user = user
        return new Promise((resolve, reject) => {
            if (!this._host || !this._port) reject('环境信息错误')
            if (!this._user || !this._user._userName || !this._user._password) reject('用户信息错误')

            if (_isWx()) {
                // 创建一个 WebSocket 连接
                wx.connectSocket({
                    url: `ws://${this._host}:${this._port}/ws`
                })

                // 监听 WebSocket 连接打开事件
                wx.onSocketOpen((res) => {
                    this.socketOpen = true
                    this.ws.onopen()
                })

                wx.onSocketMessage(res => {
                    this.ws.onmessage(res)
                    // 推送成功处理数据
                })
                Stomp.setInterval = (interval, f) => {
                    return setInterval(f, interval)
                }
                Stomp.clearInterval = (id) => {
                    return clearInterval(id)
                }
                this._client = Stomp.over(this.ws)

            } else {
                //初始化连接
                this._client = Stomp.client(`ws://${this._host}:${this._port}/ws`)
            }
            this._connectClient(resolve, reject)

        })
    }

}

class SzyxPushUser {
    constructor() {

        //用户提供
        this._tenantNo = null // 租户id
        this._userId = null // 租户下用户id
        this._deviceId = null // 浏览器唯一码

        //服务端返回
        this._account = null // 唯一id
        this._userName = null // im登录用户名
        this._password = null // im登录密码
    }

}

let _imClient = new SzyxPushClient()
let _imUser = new SzyxPushUser()

const _isWx = () => {
    return _imClient._deviceType === 6
}

const _createFingerprint = () => {
    return new Promise((resolve) => {

        Fingerprint2.get((components) => { // 参数只有回调函数时，默认浏览器指纹依据所有配置信息进行生成
            const values = components.map(function (component, index) {
                if (index === 0 && component.value) { //把微信浏览器里UA的wifi或4G等网络替换成空,不然切换网络会ID不一样
                    return component.value.replace(/\bNetType\/\w+\b/, '')
                }
                return component.value
            })
            resolve(Fingerprint2.x64hash128(values.join(''), 31))
        })

    })
}

const _connect = (res, resolve, reject) => {
    _imClient.setHostAndPort('192.168.115.137', '15774')

    _imUser._account = '56465456'

    _imUser._userName = _imUser._userId
    _imUser._password = _imUser._userId

    _imClient.connect(_imUser).then(() => resolve()).catch(error => reject(error))

}

const SzyxPush = {
    _baseUrl: 'http://192.168.115.137:8888', setBaseUrl: function (url) {
        if (url && url.startsWith('http'))
            this._baseUrl = url
    }, install: function (app, tenantNo, osType, url) {
        return this.init(tenantNo, osType, url)
    }, init: function (tenantNo, osType, url) {
        if (url && url.startsWith('http'))
            this._baseUrl = url
        console.log(`${tenantNo}::${osType}`)
        _imUser._tenantNo = tenantNo
        _imClient.setDeviceType(osType)
        return new Promise((resolve, reject) => {
            if (!tenantNo) reject('租户id不能为空')
            _createFingerprint().then(r => {
                _imUser._deviceId = r
                resolve()
            })
        })
    }, onMessage: function (callback) {
        return _imClient.onMessage(callback)
    }, unMessage: function (callId) {
        _imClient.unMessage(callId)
    }, register: function (userId) {
        return new Promise((resolve, reject) => {
            if (!userId) reject('用户id不能为空')
            _imUser._userId = userId
            if (_isWx()) {
                uni.request({
                    url: `${this._baseUrl}/hello/123`, //仅为示例，并非真实接口地址。
                    success: (res) => {
                        _connect(res.data, resolve, reject)
                    }, fail: e => {
                        reject(`注册im服务失败::${error}`)
                    }
                });

            } else {
                axios.create({
                    timeout: 60000
                }).get(`${this._baseUrl}/hello/123`)
                    .then((res) => {
                        _connect(res.data, resolve, reject)
                    })
                    .catch((error) => {
                        reject(`注册im服务失败::${error}`)
                    })
            }
        })
    }, sendTextMsg: function (userId, text, describe) {
        return _imClient.sendTextMsg(userId, text, describe)
    }
};
export default SzyxPush
